import * as geojson from 'geojson';
import * as Poly from '../poly/all';
export declare type GeoProperties = geojson.GeoJsonProperties;
export declare type GeoFeature = geojson.Feature;
export declare type GeoFeatureArray = GeoFeature[];
export declare type GeoFeatureCollection = geojson.FeatureCollection;
export declare type GeoCentroidMap = {
    [geoid: string]: {
        x: number;
        y: number;
    };
};
export interface NormalizeOptions {
    joinPolygons?: boolean;
    checkRewind?: boolean;
    ensureID?: boolean;
}
export declare function geoEnsureID(col: GeoFeatureCollection, options?: NormalizeOptions): void;
export declare function geoNormalizeFeature(f: any, options?: NormalizeOptions): GeoFeature;
export declare function geoNormalizeCollection(col: GeoFeatureCollection, options?: NormalizeOptions): GeoFeatureCollection;
export declare function geoCollectionToMap(col: GeoFeatureCollection): GeoFeatureMap;
export declare function geoMapToCollection(map: GeoFeatureMap): GeoFeatureCollection;
export declare function geoMapToCollectionNonNull(map: GeoFeatureMap): GeoFeatureCollection;
export declare function geoCollectionToTopo(col: GeoFeatureCollection): Poly.Topo;
export declare function geoCollectionToTopoNonNull(col: GeoFeatureCollection): Poly.Topo;
export declare function geoTopoToCollection(topo: Poly.Topo): GeoFeatureCollection;
export declare function geoTopoToCollectionNonNull(topo: Poly.Topo): GeoFeatureCollection;
export interface GeoFeatureMap {
    [id: string]: GeoFeature;
}
export declare type FeatureFunc = (f: GeoFeature) => void;
interface GeoEntry {
    tag: string;
    col?: GeoFeatureCollection;
    map?: GeoFeatureMap;
    topo?: Poly.Topo;
}
declare type GeoEntryMap = {
    [tag: string]: GeoEntry;
};
export declare function geoEqual(m1: GeoMultiCollection, m2: GeoMultiCollection): boolean;
export declare function geoMapEqual(m1: GeoFeatureMap, m2: GeoFeatureMap): boolean;
export declare class GeoMultiCollection {
    entries: GeoEntryMap;
    all: GeoEntry;
    hidden: any;
    stamp: number;
    constructor(tag?: string, topo?: Poly.Topo, col?: GeoFeatureCollection, map?: GeoFeatureMap);
    empty(): void;
    get nEntries(): number;
    nthEntry(n: number): GeoEntry;
    add(tag: string, topo: Poly.Topo, col: GeoFeatureCollection, map: GeoFeatureMap): void;
    addMulti(multi: GeoMultiCollection): void;
    remove(tag: string): void;
    _onChange(): void;
    _col(e: GeoEntry): GeoFeatureCollection;
    _map(e: GeoEntry): GeoFeatureMap;
    _topo(e: GeoEntry): Poly.Topo;
    colOf(tag: string): GeoFeatureCollection;
    mapOf(tag: string): GeoFeatureMap;
    topoOf(tag: string): Poly.Topo;
    forEachEntry(cb: (e: GeoEntry) => void): void;
    allCol(): GeoFeatureCollection;
    allMap(): GeoFeatureMap;
    allTopo(): Poly.Topo;
    hide(id: any): void;
    show(id: any): void;
    showAll(): void;
    get length(): number;
    nthFeature(n: number): GeoFeature;
    nthFilteredFeature(n: number, cb: (f: GeoFeature) => boolean): GeoFeature;
    forEach(cb: FeatureFunc): void;
    map(cb: (f: GeoFeature) => GeoFeature): GeoFeature[];
    isHidden(id: string): boolean;
    find(id: string): GeoFeature;
    filter(test: (f: GeoFeature) => boolean): GeoMultiCollection;
}
export declare enum geoIntersectOptions {
    Intersects = 0,
    Bounds = 1,
    BoundsCenter = 2
}
export declare function geoIntersect(multi: GeoMultiCollection, bbox: Poly.BoundBox, opt: geoIntersectOptions): GeoMultiCollection;
export {};
