export interface Coder {
    encoder: {
        encode: (s: string) => Uint8Array;
    };
    decoder: {
        decode: (u8: Uint8Array) => string;
    };
}
export declare function s2u8(coder: Coder, s: string): Uint8Array;
export declare function u82s(coder: Coder, u8: Uint8Array, offset: number, n: number): string;
interface Fragment {
    count: number;
    used: number;
    offset: number;
    s: string;
    u8?: Uint8Array;
}
declare type FragmentTable = {
    [frag: string]: Fragment;
};
declare type FragmentPair = {
    f1: Fragment;
    f2: Fragment;
    offset?: number;
};
declare class ValueTable {
    coder: Coder;
    ab: ArrayBuffer;
    u8: Uint8Array;
    u32: Uint32Array;
    constructor(coder: Coder);
    static fromBuffer(coder: Coder, ab: ArrayBuffer, offset: number, length: number): ValueTable;
    fromOffset(offset: number): string;
}
declare class ValueTableBuilder {
    coder: Coder;
    values: string[];
    fragments: FragmentTable;
    pairs: {
        [value: string]: FragmentPair;
    };
    ab: ArrayBuffer;
    u8: Uint8Array;
    u32: Uint32Array;
    constructor(coder: Coder);
    static fromStrings(coder: Coder, values: string[]): ValueTableBuilder;
    validateStrings(values: string[]): void;
    addFragment(s: string): void;
    extractFragments(s: string): void;
    pickFragments(s: string): void;
    toOffset(s: string): number;
    toBinary(): void;
}
export declare type StringMap = {
    [key: string]: string;
};
interface UnpackedNode {
    suffix: Uint8Array;
    value: number;
    bytes: number[] | null;
    nodes: UnpackedNode[] | null;
}
export declare class BinTrie {
    coder: Coder;
    ab: ArrayBuffer;
    vt: ValueTable;
    u8: Uint8Array;
    i32: Int32Array;
    constructor(coder: Coder);
    static fromBuffer(coder: Coder, ab: ArrayBuffer): BinTrie;
    get(key: string): string;
    forEach(cb: (k: string, v: string) => void): void;
}
export interface BinTrieOptions {
    dedup?: boolean;
    verbose?: boolean;
}
export declare class BinTrieBuilder {
    options: BinTrieOptions;
    coder: Coder;
    root: UnpackedNode;
    vtb: ValueTableBuilder;
    vt: ValueTable;
    ab: ArrayBuffer;
    u8: Uint8Array;
    i32: Int32Array;
    constructor(coder: Coder, options?: BinTrieOptions);
    nodeCount(node: UnpackedNode): number;
    nodeBranching(node: UnpackedNode, counts: number[]): number;
    nodeValueCount(node: UnpackedNode): number;
    nodeByteLength(node: UnpackedNode): number;
    addString(s: string, value: number): void;
    dedup(node: UnpackedNode): void;
    getUnpacked(key: string): string;
    packNode(node: UnpackedNode, byteOffset: number): number;
    toBinary(): void;
    static fromMap(coder: Coder, o: StringMap, options?: BinTrieOptions): BinTrie;
}
export {};
